@@ -1,5 +1,5 @@
 require 'omniauth/strategies/oauth2'
-require 'base64'
+require 'omniauth/facebook/signed_request'
 require 'openssl'
 require 'rack/utils'
 require 'uri'
@@ -8,27 +8,26 @@ module OmniAuth
   module Strategies
     class Facebook < OmniAuth::Strategies::OAuth2
       class NoAuthorizationCodeError < StandardError; end
-      class UnknownSignatureAlgorithmError < NotImplementedError; end
 
       DEFAULT_SCOPE = 'email'
-      SUPPORTED_ALGORITHM = 'HMAC-SHA256'
+      DEFAULT_FACEBOOK_API_VERSION = 'v19.0'.freeze
 
       option :client_options, {
-        :site => 'https://graph.facebook.com',
-        :authorize_url => "https://www.facebook.com/dialog/oauth",
-        :token_url => 'oauth/access_token'
-      }
-
-      option :token_params, {
-        :parse => :query
+        site: "https://graph.facebook.com/#{DEFAULT_FACEBOOK_API_VERSION}",
+        authorize_url: "https://www.facebook.com/#{DEFAULT_FACEBOOK_API_VERSION}/dialog/oauth",
+        token_url: 'oauth/access_token'
       }
 
       option :access_token_options, {
-        :header_format => 'OAuth %s',
-        :param_name => 'access_token'
+        header_format: 'OAuth %s',
+        param_name: 'access_token'
       }
 
-      option :authorize_options, [:scope, :display, :auth_type]
+      option :authorization_code_from_signed_request_in_cookie, nil
+
+      option :authorize_options, [:scope, :display, :auth_type, :config_id]
+
+      option :secure_image_url, true
 
       uid { raw_info['id'] }
 
@@ -61,11 +60,11 @@ module OmniAuth
       end
 
       def info_options
-        params = {:appsecret_proof => appsecret_proof}
-        params.merge!({:fields => options[:info_fields]}) if options[:info_fields]
-        params.merge!({:locale => options[:locale]}) if options[:locale]
+        params = {appsecret_proof: appsecret_proof}
+        params.merge!({fields: (options[:info_fields] || 'name,email')})
+        params.merge!({locale: options[:locale]}) if options[:locale]
 
-        { :params => params }
+        { params: params }
       end
 
       def callback_phase
@@ -74,7 +73,7 @@ module OmniAuth
         end
       rescue NoAuthorizationCodeError => e
         fail!(:no_authorization_code, e)
-      rescue UnknownSignatureAlgorithmError => e
+      rescue OmniAuth::Facebook::SignedRequest::UnknownSignatureAlgorithmError => e
         fail!(:unknown_signature_algorithm, e)
       end
 
@@ -82,10 +81,11 @@ module OmniAuth
       #      phase and it must match during the access_token phase:
       #      https://github.com/facebook/facebook-php-sdk/blob/master/src/base_facebook.php#L477
       def callback_url
-        if @authorization_code_from_signed_request_in_cookie
+        if options.authorization_code_from_signed_request_in_cookie
           ''
         else
-          options[:callback_url] || super
+          # Fixes regression in omniauth-oauth2 v1.4.0 by https://github.com/intridea/omniauth-oauth2/commit/85fdbe117c2a4400d001a6368cc359d88f40abc7
+          options[:callback_url] || (full_host + callback_path)
         end
       end
 
@@ -93,13 +93,13 @@ module OmniAuth
         options.access_token_options.inject({}) { |h,(k,v)| h[k.to_sym] = v; h }
       end
 
-      # You can pass +display+, +scope+, or +auth_type+ params to the auth request, if you need to set them dynamically.
+      # You can pass +display+, +scope+, +auth_type+ or +config_id+ params to the auth request, if you need to set them dynamically.
       # You can also set these options in the OmniAuth config :authorize_params option.
       #
       # For example: /auth/facebook?display=popup
       def authorize_params
         super.tap do |params|
-          %w[display scope auth_type].each do |v|
+          %w[display scope auth_type config_id].each do |v|
             if request.params[v]
               params[v.to_sym] = request.params[v]
             end
@@ -120,7 +120,7 @@ module OmniAuth
       private
 
       def signed_request_from_cookie
-        @signed_request_from_cookie ||= raw_signed_request_from_cookie && parse_signed_request(raw_signed_request_from_cookie)
+        @signed_request_from_cookie ||= raw_signed_request_from_cookie && OmniAuth::Facebook::SignedRequest.parse(raw_signed_request_from_cookie, client.secret)
       end
 
       def raw_signed_request_from_cookie
@@ -136,7 +136,7 @@ module OmniAuth
           yield
         elsif code_from_signed_request = signed_request_from_cookie && signed_request_from_cookie['code']
           request.params['code'] = code_from_signed_request
-          @authorization_code_from_signed_request_in_cookie = true
+          options.authorization_code_from_signed_request_in_cookie = true
           # NOTE The code from the signed fbsr_XXX cookie is set by the FB JS SDK will confirm that the identity of the
           #      user contained in the signed request matches the user loading the app.
           original_provider_ignores_state = options.provider_ignores_state
@@ -145,7 +145,7 @@ module OmniAuth
             yield
           ensure
             request.params.delete('code')
-            @authorization_code_from_signed_request_in_cookie = false
+            options.authorization_code_from_signed_request_in_cookie = false
             options.provider_ignores_state = original_provider_ignores_state
           end
         else
@@ -160,42 +160,19 @@ module OmniAuth
         end
       end
 
-      def parse_signed_request(value)
-        signature, encoded_payload = value.split('.')
-        return if signature.nil?
-
-        decoded_hex_signature = base64_decode_url(signature)
-        decoded_payload = MultiJson.decode(base64_decode_url(encoded_payload))
-
-        unless decoded_payload['algorithm'] == SUPPORTED_ALGORITHM
-          raise UnknownSignatureAlgorithmError, "unknown algorithm: #{decoded_payload['algorithm']}"
-        end
-
-        if valid_signature?(client.secret, decoded_hex_signature, encoded_payload)
-          decoded_payload
-        end
-      end
-
-      def valid_signature?(secret, signature, payload, algorithm = OpenSSL::Digest::SHA256.new)
-        OpenSSL::HMAC.digest(algorithm, secret, payload) == signature
-      end
-
-      def base64_decode_url(value)
-        value += '=' * (4 - value.size.modulo(4))
-        Base64.decode64(value.tr('-_', '+/'))
-      end
-
       def image_url(uid, options)
         uri_class = options[:secure_image_url] ? URI::HTTPS : URI::HTTP
         site_uri = URI.parse(client.site)
-        url = uri_class.build({:host => site_uri.host, :path => "#{site_uri.path}/#{uid}/picture"})
+        url = uri_class.build({host: site_uri.host, path: "#{site_uri.path}/#{uid}/picture"})
+        query = { access_token: access_token.token }
 
-        query = if options[:image_size].is_a?(String) || options[:image_size].is_a?(Symbol)
-          { :type => options[:image_size] }
+        if options[:image_size].is_a?(String) || options[:image_size].is_a?(Symbol)
+          query[:type] = options[:image_size]
         elsif options[:image_size].is_a?(Hash)
-          options[:image_size]
+          query.merge!(options[:image_size])
         end
-        url.query = Rack::Utils.build_query(query) if query
+
+        url.query = Rack::Utils.build_query(query)
 
         url.to_s
       end
