@@ -1,6 +1,6 @@
 require 'bundler/setup'
 require 'minitest/autorun'
-require 'mocha/setup'
+require 'mocha/minitest'
 require 'omniauth/strategies/facebook'
 
 OmniAuth.config.test_mode = true
@@ -36,10 +36,14 @@ class StrategyTestCase < TestCase
     @request.stubs(:params).returns({})
     @request.stubs(:cookies).returns({})
     @request.stubs(:env).returns({})
+    @request.stubs(:scheme).returns({})
     @request.stubs(:ssl?).returns(false)
 
     @client_id = '123'
     @client_secret = '53cr3tz'
+    @options = {}
+
+    @facebook_api_version = OmniAuth::Strategies::Facebook::DEFAULT_FACEBOOK_API_VERSION
   end
 
   def strategy
@@ -52,4 +56,4 @@ class StrategyTestCase < TestCase
   end
 end
 
-Dir[File.expand_path('../support/**/*', __FILE__)].each &method(:require)
+Dir[File.expand_path('../support/**/*', __FILE__)].each(&method(:require))
