@@ -9,11 +9,11 @@ end
 
 class ClientTest < StrategyTestCase
   test 'has correct Facebook site' do
-    assert_equal 'https://graph.facebook.com/v2.6', strategy.client.site
+    assert_equal "https://graph.facebook.com/#{@facebook_api_version}", strategy.client.site
   end
 
   test 'has correct authorize url' do
-    assert_equal 'https://www.facebook.com/v2.6/dialog/oauth', strategy.client.options[:authorize_url]
+    assert_equal "https://www.facebook.com/#{@facebook_api_version}/dialog/oauth", strategy.client.options[:authorize_url]
   end
 
   test 'has correct token url with versioning' do
@@ -26,10 +26,11 @@ end
 class CallbackUrlTest < StrategyTestCase
   test "returns the default callback url (omitting querystring)" do
     url_base = 'http://auth.request.com'
+    script_name = '/script_name'
     @request.stubs(:url).returns("#{url_base}/some/page")
-    strategy.stubs(:script_name).returns('') # as not to depend on Rack env
+    strategy.stubs(:script_name).returns(script_name) # as not to depend on Rack env
     strategy.stubs(:query_string).returns('?foo=bar')
-    assert_equal "#{url_base}/auth/facebook/callback", strategy.callback_url
+    assert_equal "#{url_base}#{script_name}/auth/facebook/callback", strategy.callback_url
   end
 
   test "returns path from callback_path option (omitting querystring)" do
@@ -60,6 +61,12 @@ class AuthorizeParamsTest < StrategyTestCase
     assert_equal 'touch', strategy.authorize_params[:display]
   end
 
+  test 'includes config_id parameter from request when present' do
+    @request.stubs(:params).returns({ 'config_id' => '000111222' })
+    assert strategy.authorize_params.is_a?(Hash)
+    assert_equal '000111222', strategy.authorize_params[:config_id]
+  end
+
   test 'includes auth_type parameter from request when present' do
     @request.stubs(:params).returns({ 'auth_type' => 'reauthenticate' })
     assert strategy.authorize_params.is_a?(Hash)
@@ -95,41 +102,67 @@ class UidTest < StrategyTestCase
 end
 
 class InfoTest < StrategyTestCase
-  test 'returns the secure facebook avatar url when `secure_image_url` option is specified' do
+  def setup
+    super
+    @access_token = stub('OAuth2::AccessToken')
+    @access_token.stubs(:token).returns('test_access_token')
+  end
+
+  test 'returns the secure facebook avatar url when `secure_image_url` option is set to true' do
     @options = { secure_image_url: true }
     raw_info = { 'name' => 'Fred Smith', 'id' => '321' }
     strategy.stubs(:raw_info).returns(raw_info)
-    assert_equal 'https://graph.facebook.com/v2.6/321/picture', strategy.info['image']
+    strategy.stubs(:access_token).returns(@access_token)
+    assert_equal "https://graph.facebook.com/#{@facebook_api_version}/321/picture?access_token=test_access_token", strategy.info['image']
+  end
+
+  test 'returns the non-ssl facebook avatar url when `secure_image_url` option is set to false' do
+    @options = { secure_image_url: false }
+    raw_info = { 'name' => 'Fred Smith', 'id' => '321' }
+    strategy.stubs(:raw_info).returns(raw_info)
+    strategy.stubs(:access_token).returns(@access_token)
+    assert_equal "http://graph.facebook.com/#{@facebook_api_version}/321/picture?access_token=test_access_token", strategy.info['image']
+  end
+
+  test 'returns the secure facebook avatar url when `secure_image_url` option is omitted' do
+    raw_info = { 'name' => 'Fred Smith', 'id' => '321' }
+    strategy.stubs(:raw_info).returns(raw_info)
+    strategy.stubs(:access_token).returns(@access_token)
+    assert_equal "https://graph.facebook.com/#{@facebook_api_version}/321/picture?access_token=test_access_token", strategy.info['image']
   end
 
   test 'returns the image_url based of the client site' do
     @options = { secure_image_url: true, client_options: {site: "https://blah.facebook.com/v2.2"}}
     raw_info = { 'name' => 'Fred Smith', 'id' => '321' }
     strategy.stubs(:raw_info).returns(raw_info)
-    assert_equal 'https://blah.facebook.com/v2.2/321/picture', strategy.info['image']
+    strategy.stubs(:access_token).returns(@access_token)
+    assert_equal "https://blah.facebook.com/v2.2/321/picture?access_token=test_access_token", strategy.info['image']
   end
 
   test 'returns the image with size specified in the `image_size` option' do
     @options = { image_size: 'normal' }
     raw_info = { 'name' => 'Fred Smith', 'id' => '321' }
     strategy.stubs(:raw_info).returns(raw_info)
-    assert_equal 'http://graph.facebook.com/v2.6/321/picture?type=normal', strategy.info['image']
+    strategy.stubs(:access_token).returns(@access_token)
+    assert_equal "https://graph.facebook.com/#{@facebook_api_version}/321/picture?access_token=test_access_token&type=normal", strategy.info['image']
   end
 
   test 'returns the image with size specified as a symbol in the `image_size` option' do
     @options = { image_size: :normal }
     raw_info = { 'name' => 'Fred Smith', 'id' => '321' }
     strategy.stubs(:raw_info).returns(raw_info)
-    assert_equal 'http://graph.facebook.com/v2.6/321/picture?type=normal', strategy.info['image']
+    strategy.stubs(:access_token).returns(@access_token)
+    assert_equal "https://graph.facebook.com/#{@facebook_api_version}/321/picture?access_token=test_access_token&type=normal", strategy.info['image']
   end
 
   test 'returns the image with width and height specified in the `image_size` option' do
     @options = { image_size: { width: 123, height: 987 } }
     raw_info = { 'name' => 'Fred Smith', 'id' => '321' }
     strategy.stubs(:raw_info).returns(raw_info)
+    strategy.stubs(:access_token).returns(@access_token)
     assert_match 'width=123', strategy.info['image']
     assert_match 'height=987', strategy.info['image']
-    assert_match 'http://graph.facebook.com/v2.6/321/picture?', strategy.info['image']
+    assert_match "https://graph.facebook.com/#{@facebook_api_version}/321/picture?access_token=test_access_token", strategy.info['image']
   end
 end
 
@@ -138,6 +171,10 @@ class InfoTestOptionalDataPresent < StrategyTestCase
     super
     @raw_info ||= { 'name' => 'Fred Smith' }
     strategy.stubs(:raw_info).returns(@raw_info)
+
+    access_token = stub('OAuth2::AccessToken')
+    access_token.stubs(:token).returns('test_access_token')
+    strategy.stubs(:access_token).returns(access_token)
   end
 
   test 'returns the name' do
@@ -176,7 +213,7 @@ class InfoTestOptionalDataPresent < StrategyTestCase
 
   test 'returns the facebook avatar url' do
     @raw_info['id'] = '321'
-    assert_equal 'http://graph.facebook.com/v2.6/321/picture', strategy.info['image']
+    assert_equal "https://graph.facebook.com/#{@facebook_api_version}/321/picture?access_token=test_access_token", strategy.info['image']
   end
 
   test 'returns the Facebook link as the Facebook url' do
@@ -215,6 +252,10 @@ class InfoTestOptionalDataNotPresent < StrategyTestCase
     super
     @raw_info ||= { 'name' => 'Fred Smith' }
     strategy.stubs(:raw_info).returns(@raw_info)
+
+    access_token = stub('OAuth2::AccessToken')
+    access_token.stubs(:token).returns('test_access_token')
+    strategy.stubs(:access_token).returns(access_token)
   end
 
   test 'has no email key' do
@@ -258,7 +299,7 @@ class RawInfoTest < StrategyTestCase
     @options = {appsecret_proof: @appsecret_proof, fields: 'name,email'}
   end
 
-  test 'performs a GET to https://graph.facebook.com/v2.6/me' do
+  test "performs a GET to https://graph.facebook.com/#{@facebook_api_version}/me" do
     strategy.stubs(:appsecret_proof).returns(@appsecret_proof)
     strategy.stubs(:access_token).returns(@access_token)
     params = {params: @options}
@@ -266,7 +307,7 @@ class RawInfoTest < StrategyTestCase
     strategy.raw_info
   end
 
-  test 'performs a GET to https://graph.facebook.com/v2.6/me with locale' do
+  test "performs a GET to https://graph.facebook.com/#{@facebook_api_version}/me with locale" do
     @options.merge!({ locale: 'cs_CZ' })
     strategy.stubs(:access_token).returns(@access_token)
     strategy.stubs(:appsecret_proof).returns(@appsecret_proof)
@@ -275,7 +316,7 @@ class RawInfoTest < StrategyTestCase
     strategy.raw_info
   end
 
-  test 'performs a GET to https://graph.facebook.com/v2.6/me with info_fields' do
+  test "performs a GET to https://graph.facebook.com/#{@facebook_api_version}/me with info_fields" do
     @options.merge!({info_fields: 'about'})
     strategy.stubs(:access_token).returns(@access_token)
     strategy.stubs(:appsecret_proof).returns(@appsecret_proof)
@@ -284,7 +325,7 @@ class RawInfoTest < StrategyTestCase
     strategy.raw_info
   end
 
-  test 'performs a GET to https://graph.facebook.com/v2.6/me with default info_fields' do
+  test "performs a GET to https://graph.facebook.com/#{@facebook_api_version}/me with default info_fields" do
     strategy.stubs(:access_token).returns(@access_token)
     strategy.stubs(:appsecret_proof).returns(@appsecret_proof)
     params = {params: {appsecret_proof: @appsecret_proof, fields: 'name,email'}}
@@ -393,7 +434,7 @@ end
 
 module SignedRequestHelpers
   def signed_request(payload, secret)
-    encoded_payload = base64_encode_url(MultiJson.encode(payload))
+    encoded_payload = base64_encode_url(JSON.dump(payload))
     encoded_signature = base64_encode_url(signature(encoded_payload, secret))
     [encoded_signature, encoded_payload].join('.')
   end
@@ -452,7 +493,7 @@ module SignedRequestTests
     end
 
     test 'empty param' do
-      assert_equal nil, strategy.send(:signed_request_from_cookie)
+      assert_nil strategy.send(:signed_request_from_cookie)
     end
   end
 
